use std::fmt::Display;

use openidconnect::AccessToken;
use axum::response::{
	IntoResponse,
	Redirect,
};
use reqwest::StatusCode;

#[derive(Debug)]
pub enum AuthError {
	OpenIdConfig(openidconnect::ConfigurationError),
	OpenIdRequest(String),
	TokenNotActive(AccessToken),
}

impl Display for AuthError {
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		match self {
			AuthError::OpenIdConfig(e) => f.write_fmt(format_args!("OpenID Connect configuration error: {:?}", e)),
			AuthError::OpenIdRequest(s) => f.write_fmt(format_args!("{}", s)),
			AuthError::TokenNotActive(t) => f.write_fmt(format_args!("Token {} is not active", t.secret())),
		}
	}
}

impl std::error::Error for AuthError {}

impl From<openidconnect::ConfigurationError> for AuthError {
	fn from(e: openidconnect::ConfigurationError) -> Self {
		Self::OpenIdConfig(e)
	}
}

impl From<String> for AuthError {
	fn from(s: String) -> Self {
		Self::OpenIdRequest(s)
	}
}

impl From<AccessToken> for AuthError {
	fn from(t: AccessToken) -> Self {
		Self::TokenNotActive(t)
	}
}


impl IntoResponse for AuthError {
	fn into_response(self) -> axum::response::Response {
		match self {
			AuthError::TokenNotActive(_) => Redirect::to("/openid/login").into_response(),
			_ =>	(
					StatusCode::INTERNAL_SERVER_ERROR,
					format!("Authentication error: {}", self),
			).into_response(),
		}
	}
}
