use std::vec::Vec;
use std::fmt::Display;

use serde::{
	Deserialize,
	Serialize,
};
use axum::response::IntoResponse;
use reqwest::StatusCode;

#[derive(Debug, Serialize, Deserialize)]
pub struct PowerDnsApiError {
	pub error: String,
	pub errors: Option<Vec<String>>,
}

#[derive(Debug)]
pub enum PowerDnsError {
	Reqwest(reqwest::Error),
	Serde(serde_json::Error),
	Api(PowerDnsApiError),
}

impl Display for PowerDnsError {
	fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
		match self {
			PowerDnsError::Reqwest(r) => f.write_fmt(format_args!("Request error on {:?}: {:?}", r.url(), r)),
			PowerDnsError::Serde(s) => f.write_fmt(format_args!("SerDe error: {:?}", s)),
			PowerDnsError::Api(a) => f.write_fmt(format_args!("API error: {:?}", a)),
		}
	}
}

impl std::error::Error for PowerDnsError {}

impl From<reqwest::Error> for PowerDnsError {
	fn from(e: reqwest::Error) -> Self {
		Self::Reqwest(e)
	}
}
impl From<serde_json::Error> for PowerDnsError {
	fn from(e: serde_json::Error) -> Self {
		Self::Serde(e)
	}
}

impl From<PowerDnsApiError> for PowerDnsError {
	fn from(e: PowerDnsApiError) -> Self {
		Self::Api(e)
	}
}
pub type PowerDnsResult<T> = Result<T, PowerDnsError>;

impl IntoResponse for PowerDnsError {
	fn into_response(self) -> axum::response::Response {
		(
			StatusCode::INTERNAL_SERVER_ERROR,
			format!("Error processing request: {}", self),
		).into_response()
	}
}

impl From<reqwest::Error> for PowerDnsApiError {
	fn from(err: reqwest::Error) -> Self {
		Self {
			error: format!("{:?}", err),
			errors: None,
		}
	}
}
