use openidconnect::reqwest::async_http_client;
use openidconnect::core::{
	CoreClient,
	CoreProviderMetadata,
};
use openidconnect::{
	ClientId,
	ClientSecret,
	IssuerUrl,
	RedirectUrl,
};
use openidconnect::TokenIntrospectionResponse;
use url::Url;
use axum::{
	async_trait,
	extract::{
		FromRef,
		FromRequestParts,
		State,
	},
	http::{
		StatusCode,
		request::Parts,
	},
	response::Redirect,
};
use axum_extra::extract::cookie::CookieJar;

use crate::AppState;

pub(crate) struct AccessToken(pub(crate) openidconnect::AccessToken);
pub(crate) struct ValidAccessToken(pub(crate) openidconnect::AccessToken);
pub(crate) type AccessTokenRejection = (StatusCode, String);

#[async_trait]
impl<P> FromRequestParts<P> for AccessToken
where
	P: Send + Sync
{
	type Rejection = AccessTokenRejection;

	async fn from_request_parts(parts: &mut Parts, state: &P) -> Result<Self, AccessTokenRejection> {
		let cookie_name = "access_token";
		let cookies = CookieJar::from_request_parts(parts, state).await.unwrap();
		let token: Option<String> = cookies
			.get(cookie_name)
			.map(|cookie| cookie.value().to_owned());
		match token {
			Some(token) => Ok(Self(openidconnect::AccessToken::new(token))),
			None => Err((
				StatusCode::BAD_REQUEST,
				format!(
					"Request is missing the '{}' cookie",
					cookie_name.to_string(),
				)),
			)
		}
	}
}

#[async_trait]
impl<P> FromRequestParts<P> for ValidAccessToken
where
	AppState: FromRef<P>,
	P: Send + Sync,
{
	type Rejection = Redirect;

	async fn from_request_parts(parts: &mut Parts, state: &P) -> Result<Self, Self::Rejection> {
		match AccessToken::from_request_parts(parts, state).await {
			Err(_) => Err(Redirect::to("/openid/login")),
			Ok(AccessToken(token)) => {
				match AppState::from_ref(state)
					.oidc_client
					.introspect(&token)
					.map_err(|_| Redirect::to("/openid/login"))?
					.request_async(async_http_client)
					.await
					.map_err(|_| Redirect::to("/openid/login")) {
					Ok(t) => match t.active() {
						true => Ok(ValidAccessToken(token)),
						false => Err(Redirect::to("/openid/login")),
					},
					Err(e) => Err(e),
				}
			}
		}
	}
}

pub async fn create_client(issuer: Url, id: String, secret: String, redirect_url: Url) -> CoreClient {
	let issuer_url = IssuerUrl::from_url(issuer);
	let metadata = CoreProviderMetadata::discover_async(issuer_url, async_http_client)
		.await
		.unwrap_or_else(|err| {
			println!("Unable to discover provider metadata: {}", err);
			unreachable!();
		});
	CoreClient::from_provider_metadata(
		metadata,
		ClientId::new(id),
		Some(ClientSecret::new(secret)),
	)
	.set_redirect_uri(RedirectUrl::from_url(redirect_url))
}
