use std::vec::Vec;

use url::Url;
use serde::{
	de::DeserializeOwned,
};
use reqwest::{
	Client,
	Method,
	RequestBuilder,
	StatusCode,
	Response,
};

use crate::model::tsigkey::TsigKey;
use crate::error::powerdns::{
	PowerDnsError,
	PowerDnsResult,
	PowerDnsApiError,
};

#[derive(Clone)]
pub struct PowerDnsApi {
	api_url: Url,
	api_key: String,
	server_id: String,
	http_client: Client
}

// some comment
impl PowerDnsApi {
	pub async fn new(api_url: Url, api_key: String, server_id: String) -> Self {
		PowerDnsApi {
			api_url,
			api_key,
			server_id,
			http_client: Client::new(),
		}
	}

	pub async fn list_tsig_keys(self) -> PowerDnsResult<Vec<TsigKey>> {
		Self::transform_response::<Vec<TsigKey>>(
			self.get_request_for(Method::GET, "tsigkeys".to_string()).send().await?,
			StatusCode::OK,
		).await
	}

	pub async fn get_tsig_key(self, name: String) -> PowerDnsResult<TsigKey> {
		Self::transform_response::<TsigKey>(
			self.get_request_for(Method::GET, format!("tsigkeys/{}", name)).send().await?,
			StatusCode::OK,
		).await
	}

	pub async fn add_tsig_key(self, key: TsigKey) -> PowerDnsResult<TsigKey> {
		match key.secret {
			Some(ref _secret) => Self::transform_response::<TsigKey>(
				self.get_request_for(
					Method::PUT,
					format!("tsigkeys/{}", key.name),
				).json(&key).send().await?,
				StatusCode::OK,
			).await,
			None => Self::transform_response::<TsigKey>(
				self.get_request_for(Method::POST, "tsigkeys".to_string())
					.json(&key).send().await?,
				StatusCode::CREATED,
			).await,
		}
	}

	pub async fn delete_tsig_key(self, name: String) -> PowerDnsResult<()> {
		Self::transform_response::<()>(
			self.get_request_for(Method::DELETE, format!("tsigkeys/{}", name)).send().await?,
			StatusCode::NO_CONTENT,
		).await
	}

	fn get_request_for(self, method: Method, endpoint: String) -> RequestBuilder {
		let url = format!("{}api/v1/servers/{}/{}", self.api_url, self.server_id, endpoint);
		println!("Requesting {} {}", method, url);
		self.http_client.request(
			method,
			url,
		)
		.header("X-API-Key", self.api_key)
	}

	async fn transform_response<T: DeserializeOwned>(res: Response, expected: StatusCode) -> PowerDnsResult<T> {
		println!("Received response with statuscode={}", res.status());
		let unknown_status_error = |code: StatusCode| -> PowerDnsError {PowerDnsApiError {
			error: format!("Unexpected server status code: {}", code),
			errors: None, 
		}.into()};
		// TODO: Check if status_code is the 'correct' one for the endpoint
		match res.status() {
			StatusCode::OK |
			StatusCode::CREATED |
			StatusCode::NO_CONTENT => {
				if res.status() == expected { Ok(res.json().await?) } else { Err(unknown_status_error(res.status())) }
			},
			StatusCode::BAD_REQUEST |
			StatusCode::NOT_FOUND |
			StatusCode::UNPROCESSABLE_ENTITY |
			StatusCode::INTERNAL_SERVER_ERROR => Err(PowerDnsError::Api(res.json::<PowerDnsApiError>().await?)),
			_ => Err(unknown_status_error(res.status()).into()),
		}
	}
}
